import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { ExerciseService } from 'app/exercise/service/exercise.service';
import { ArtemisExerciseInfo } from 'app/shared/model/artemis-exercise-info.model';
import { JhiAlertService } from 'ng-jhipster';
import { Person } from 'app/shared/model/person.model';
import { IExerciseType } from 'app/shared/model/exercise.model';
import { GitlabService } from 'app/shared/gitlab/gitlab.service';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';
import { GitlabPathSelectorComponent } from 'app/shared/gitlab/gitlab-path-selector/gitlab-path-selector.component';
import { GitlabGroup } from 'app/shared/model/gitlab-group.model';

@Component({
  selector: 'jhi-import',
  templateUrl: './exercise-import.component.html',
  styleUrls: ['./exercise-import.component.scss'],
})
export class ExerciseImportComponent implements OnInit {
  exerciseToken: string | undefined;
  exerciseInfo: ArtemisExerciseInfo | undefined;

  newCreator = { name: '', email: '', affiliation: '' } as Person;
  selectedGitlabGroup: GitlabGroup | undefined;
  isLoading = false;
  isImporting = false;

  constructor(
    private route: ActivatedRoute,
    private exerciseService: ExerciseService,
    private jhiAlertService: JhiAlertService,
    private gitlabService: GitlabService,
    private modalService: NgbModal
  ) {}

  ngOnInit() {
    this.isLoading = true;
    this.exerciseToken = this.route.snapshot.params['token'];
    if (this.exerciseToken) {
      this.exerciseService.importExerciseInfoFromArtemis(this.exerciseToken).subscribe(
        data => {
          this.exerciseInfo = data;
          this.exerciseInfo.programmingLanguage = data['programmingLanguage'];
          this.exerciseInfo.programmingLanguage = this.exerciseInfo.programmingLanguage.map(programmingLanguage =>
            this.toTitleCase(programmingLanguage)
          );
          this.exerciseInfo.creator = data['creator'];
          if (this.exerciseInfo.language === undefined || this.exerciseInfo.language === null || this.exerciseInfo.language.length === 0) {
            this.exerciseInfo.language = ['english']; // default value if none is set
          }
          if (this.exerciseInfo.source === undefined) {
            this.route.queryParams.subscribe(params => {
              this.exerciseInfo!.source = params['callback'];
            });
          }
        },
        error => {
          this.jhiAlertService.error('', { error: error.error });
        }
      );
    }
    this.isLoading = false;
  }

  /**
   * Actual import function submitting import form to server
   */
  import() {
    if (this.exerciseInfo!.difficulty !== null) {
      this.exerciseInfo!.difficulty = this.exerciseInfo!.difficulty?.toLowerCase();
    }
    this.mapExerciseInfoValues();
    this.openGitlabPathSelectorModal().then(gitlabGroup => {
      if (gitlabGroup !== undefined) {
        this.isLoading = true;
        this.isImporting = true;
        this.exerciseService.submitExerciseInfoForImport(this.exerciseInfo!, this.exerciseToken!, gitlabGroup.id).subscribe(
          () => {
            window.location.href = this.route.snapshot.queryParams['callback'];
          },
          error => {
            this.jhiAlertService.error(error.error.detail);
            window.scroll(0, 0);
            this.isLoading = false;
            this.isImporting = false;
          }
        );
      }
    });
    this.isLoading = false;
    this.isImporting = false;
  }

  /**
   * Utility function used to open the Gitlab selector modal
   */
  openGitlabPathSelectorModal() {
    const modalRef = this.modalService.open(GitlabPathSelectorComponent, { size: 'xl', backdrop: 'static' });
    return modalRef.result.then(group => {
      this.selectedGitlabGroup = group;
      return group;
    });
  }

  /**
   * Validation function for button-groups.
   * At least one value must be set.
   * @return true if invalid and false if valid
   */
  hasValidationErrors() {
    if (this.exerciseInfo?.keyword === undefined || this.exerciseInfo?.language === undefined) {
      return true;
    }

    if (!(this.exerciseInfo.keyword.length > 0)) {
      return true;
    }

    if (!(this.exerciseInfo.language.length > 0)) {
      return true;
    }

    return false;
  }

  /**
   * Used to map specific attributes of
   * the currently imported exercise to correct metadata values
   */
  mapExerciseInfoValues() {
    let index = this.exerciseInfo!.language.indexOf('english');
    if (index !== -1) {
      this.exerciseInfo!.language[index] = 'en';
    }
    index = this.exerciseInfo!.language.indexOf('german');
    if (index !== -1) {
      this.exerciseInfo!.language[index] = 'de';
    }
  }

  /**
   * Used to set the exercise's difficulty on
   * user's click
   * @param difficulty to set
   */
  setDifficulty(difficulty: string) {
    this.exerciseInfo!.difficulty = difficulty;
  }

  /**
   * Used to add a language to the exercise
   * @param language to add
   */
  setLanguage(language: string) {
    if (this.exerciseInfo!.language.includes(language)) {
      this.exerciseInfo!.language = this.exerciseInfo!.language.filter(lang => lang !== language);
    } else {
      this.exerciseInfo!.language.push(language);
    }
  }

  /**
   * used to set the exercise's type
   * @param type to set
   */
  setType(type: string) {
    this.exerciseInfo!.type = IExerciseType[type.toUpperCase().replace(' ', '_')];
  }

  /**
   * Used to add a new person to the creators list
   * of the exercise
   */
  addCreator() {
    if (!(this.newCreator.name === '' || this.newCreator.affiliation === '' || this.newCreator.email === '')) {
      if (this.exerciseInfo && this.exerciseInfo?.creator === undefined) {
        this.exerciseInfo.creator = [];
      }
      this.exerciseInfo?.creator.push(this.newCreator);
      this.newCreator = { name: '', email: '', affiliation: '' } as Person;
    }
  }

  /**
   * Used to remove a person from the creators list
   * of the exercise
   * @param creator
   */
  removeCreator(creator: Person) {
    this.exerciseInfo!.creator = this.exerciseInfo!.creator.filter(person => person !== creator);
  }

  /**
   * Utility function used to turn words into title case
   * @param word the word to turn into title case
   */
  private toTitleCase(word: string) {
    return word[0].toUpperCase() + word.substr(1).toLowerCase();
  }

  /**
   * Utility function used to retrieve the possible difficulties
   * for an exercise
   */
  getDifficulties() {
    return ['simple', 'medium', 'advanced'];
  }

  /**
   * Utility function used to retrieve the possible languages
   * for an exercise
   */
  getLanguages() {
    return ['english', 'german'];
  }

  /**
   * Utility function used to retrieve the possible types
   */
  getTypes() {
    return Object.keys(IExerciseType)
      .map(key => IExerciseType[key])
      .filter(type => type !== 'collection'); // no supported yet
  }

  /**
   * Utility function used to check if the given type is the same
   * as the imported exercise
   * @param type to check
   */
  isExerciseType(type: string) {
    return IExerciseType[this.exerciseInfo!.type.toUpperCase().replace(' ', '_')] === IExerciseType[type.toUpperCase().replace(' ', '_')];
  }

  /**
   * Utility function used to check if the given difficulty is
   * the same as the imported exercise
   * @param difficulty to check
   */
  isDifficulty(difficulty: string) {
    if (this.exerciseInfo!.difficulty !== undefined) {
      return this.exerciseInfo!.difficulty.toLowerCase() === difficulty.toLowerCase();
    } else {
      return false;
    }
  }

  /**
   * Utility function used to check if a given language is set
   * in the imported exercsie
   * @param language
   */
  isLanguage(language: string) {
    return this.exerciseInfo!.language !== undefined && this.exerciseInfo!.language.includes(language);
  }

  /**
   * Used to validate language input
   */
  isLanguageSet() {
    return this.exerciseInfo!.language !== undefined && this.exerciseInfo!.language.length > 0;
  }
}
